/* <stpdef> :
Sottoprogrammi per la definizione dei parametri relativi a:
   -	scelta del sistema di coordinate
   -	scelta della penna da utilizzare per il tracciamento
   -	definizione della corrispondenza penne logica - penna fisica
   -	tracciamento dei caratteri
   -	fattori di scala per la conversione a coordinate fisiche
   -	margini
   -	posizioni di tabulazione orizzontale e verticale
   -	scelta del sistema di misura degli angoli
   -    qualita' grafica del file di output (per Jpeg e affini)
*/
#include <stdio.h>
#include <math.h>
#include "mizlocdf.h"
#include "mizmsg.h"
#include "mizcmn.h"
#if MIZ_INTERATTIVA
#include "icpcmn.h"
#endif
/*  */
union mizhdr mizhdr;
extern struct lgbccs lgbccs;
extern struct lgbcge lgbcge;
extern struct lgbcio lgbcio;
extern struct lgbc2d lgbc2d;
extern struct lgbcsa lgbcsa;
extern struct lgbcsg lgbcsg;
extern struct lgbcms lgbcms;
extern struct lgbcer lgbcer;
extern struct lgbccr lgbccr;
/*  */
#if MIZ_FILE
extern struct lgfile lgfile;
#endif
/*  */
#if MIZ_INTERATTIVA
extern struct lgbccf lgbccf;
extern struct lgbcpn lgbcpn;
extern struct lggraf lggraf;
#endif
/*  */
#define ABS(z) ((z)>=0)? (z):-(z)
#define MAX(z,w) ((z)>=(w))? (z):(w)
#define MIN(z,w) ((z)<=(w))? (z):(w)
/*  */
/* Sottoprogramma angoli */
VOID angoli(int k)
{  lgbcge.kanrad=(k==0);
   if(!lgbcge.kanrad)
   {  lgbccs.angiro=360.;
   }
   else
   {  lgbccs.angiro=2.*lgbccs.pigrec;
   }
   return;
}
/* Sottoprogramma coord */
VOID coord(int k)
{  
/* Controllo delle condizioni di errore */
   if(k<=0 || k>MIZ_CRDMX)
   {  lgerr(LGE_SISTCOORD);
      return;
   }
/* Chiamata valida. definisce il sistema di coordinate
   e chiede di ridefinire i coefficienti. */
   lgbc2d.kcoord=k;
   lgbc2d.kcodef=FALSE;
   return;
}
/* Sottoprogramma penna */
VOID penna(int k)
{  unsigned char pendip;
/* Controllo delle condizioni d'errore : */
   if(k<=0 || k>MIZ_NUMPEN)
   {  lgerr(LGE_PENNA);
      return;
   }
/* Chiamata valida */
   lgbcge.penna=k;
#if MIZ_FILE
   lgwpen();
#endif
#if MIZ_INTERATTIVA
   if(lgbcpn.kpenna[k-1]>lggraf.knpen)
   {  lgbcpn.pendis=TRUE;
   }
   else
   {  lgbcpn.lpnlog=k;
      lgbcpn.kvstpn= -lgbcpn.kpenna[k-1];
      lgpen(lgbcpn.kvstpn);
      lggraf.kstpn=ABS(lggraf.kstpn);
      if(lgbcpn.kvstpn<0) lggraf.kstpn= -lggraf.kstpn;
      pendip=lgbcpn.pendis;
      lgbcpn.pendis=FALSE;
      if(pendip) lgploi(MIZ_MOV);
   }
#endif
   return;
}
/* Sottoprogramma defbat */
VOID defbat(int k)
{  
#if MIZ_INTERATTIVA
   lggraf.batch=(k!=0);
#endif
   return;
}
/* Sottoprogramma defcar */
VOID defcar(int ispo,int ispv,double ryx,double dst)
{  double ryx1,tmp;
   lgbcsa.kspor=MIN(ispo,MIZ_SPOMX);
   lgbcsa.kspor=MAX(lgbcsa.kspor,MIZ_SPOMI);
   if(lgbcsa.kspor!=ispo) lgerr(LGE_DEFCAR);
   lgbcsa.kspver=MIN(ispv,MIZ_SPVMX);
   lgbcsa.kspver=MAX(lgbcsa.kspver,MIZ_SPVMI);
   if(lgbcsa.kspver!=ispv) lgerr(LGE_DEFCAR);
   if(ryx<=lgbcsa.dstmax && ryx>=(1./lgbcsa.dstmax))
   {  ryx1=sqrt(ryx);
   }
   else
   {  lgerr(LGE_DEFCAR);
      ryx1=MIN(ryx,lgbcsa.dstmax);
      ryx1=MAX(ryx1,1./lgbcsa.dstmax);
   }
   lgbcsg.dftsc[0]=1./ryx1;
   lgbcsg.dftsc[1]=ryx1;
   tmp=ABS(dst);
   if(tmp>lgbcsa.dstmax) lgerr(LGE_DEFCAR);
   lgbcsg.dstsim=MIN(dst,lgbcsa.dstmax);
   lgbcsg.dstsim=MAX(lgbcsg.dstsim,-lgbcsa.dstmax);
#if MIZ_FILE
   lgwcar();
#endif
   return;
}
/* Sottoprogramma defcen */
VOID defcen(double gxcen,double gycen)
{  
#if MIZ_INTERATTIVA
   if(gxcen>=lgbc2d.g2lim[MIZ_GRAFPAG][0]
      	&& gxcen<=lgbc2d.g2lim[MIZ_GRAFPAG][1]
   	&& gycen>=lgbc2d.g2lim[MIZ_GRAFPAG][2]
      	&& gycen<=lgbc2d.g2lim[MIZ_GRAFPAG][3])
   {  lgbccf.gfcen[0]=gxcen;
      lgbccf.gfcen[1]=gycen;
   }
   else
   {  lgbccf.gfcen[0]
      	=.5*(lgbc2d.g2lim[MIZ_GRAFPAG][0]+lgbc2d.g2lim[MIZ_GRAFPAG][1]);
      lgbccf.gfcen[1]
      	=.5*(lgbc2d.g2lim[MIZ_GRAFPAG][2]+lgbc2d.g2lim[MIZ_GRAFPAG][3]);
      lgerr(LGE_CENTROPRZ);
   }
#endif
   return;
}
/* Sottoprogramma deffat */
VOID deffat(double fat)
{  
#if MIZ_INTERATTIVA
   if(fat>0.)
   {  lgbccf.fatscg[0]=fat;
      lgbccf.fatscg[1]=fat;
      lgbccf.kcgfdf=FALSE;
   }
   else
   {  lgerr(LGE_FATTSCALA);
   }
#endif
   return;
}
/* Sottoprogramma defftx */
VOID defftx(double fat)
{  
#if MIZ_INTERATTIVA
   if(fat>0.)
   {  lgbccf.fatscg[0]=fat;
      lgbccf.kcgfdf=FALSE;
   }
   else
   {  lgerr(LGE_FATTSCALA);
   }
#endif
   return;
}
/* Sottoprogramma deffty */
VOID deffty(double fat)
{  
#if MIZ_INTERATTIVA
   if(fat>0.)
   {  lgbccf.fatscg[1]=fat;
      lgbccf.kcgfdf=FALSE;
   }
   else
   {  lgerr(LGE_FATTSCALA);
   }
#endif
   return;
}
/* Sottoprogramma defmar */
VOID defmar(double exmin,double exmax,double eymin,double eymax)
{  double tmp;
   if(exmin<lgbc2d.g2lim[MIZ_GRAFPAG][0]
   	 || exmin>=exmax
   	 || exmax>lgbc2d.g2lim[MIZ_GRAFPAG][1]
   	 || eymin<lgbc2d.g2lim[MIZ_GRAFPAG][2]
	 || eymin>=eymax
	 || eymax>lgbc2d.g2lim[MIZ_GRAFPAG][3])
   {  lgerr(LGE_MARGINI);
      return;
   }
   lgbcsg.pagmar[0]=exmin;
   lgbcsg.pagmar[1]=exmax;
   lgbcsg.pagmar[2]=eymin;
   lgbcsg.pagmar[3]=eymax;
#if MIZ_FILE
   lgwmrg();
#endif
   return;
}
/* Sottoprogramma defsim */
VOID defsim(int isim,double dim,double rot)
{  
/* Controllo correttezza del simbolo */
   if(isim<=0 || isim>MIZ_CARMX)
   {  lgerr(LGE_SIMBOLOCENT);
      return;
   }
/* Controllo dimensioni e registrazione */
   lgbc2d.dimsp=ABS(dim);
   lgbc2d.dimsp=MIN(lgbc2d.dimsp,MIZ_DSIMX);
   if(lgbc2d.dimsp!=dim) lgerr(LGE_DIMCARATT);
   lgbc2d.rotsp=fmod(rot,lgbccs.angiro);
   if(lgbc2d.rotsp<0) lgbc2d.rotsp+=lgbccs.angiro;
   if(!lgbcge.kanrad) lgbc2d.rotsp*=lgbccs.grarad;
   lgbc2d.cosp=cos(lgbc2d.rotsp);
   lgbc2d.sesp=sin(lgbc2d.rotsp);
   lgbc2d.knsp=isim;
#if MIZ_FILE
   lgwsmc();
#endif
   return;
}
/* Sottoprogramma deftbo */
VOID deftbo(DOUBLE tab[],int n)
{  if((ABS(n))>MIZ_TABOR) goto errore;
   if(n==0)
   {  int i;
/* Azzeramento delle tabelle di tabulazione */
      for(i=0;i<MIZ_TABOR;lgbcsa.tbor[i++]=0.);
   }
   else
   {  int idf,i;
      double dtb;
      if(n<0)
      {  
/* n<0 : Riempimento delle prime -N posizioni in modo che 
   siano equidistanziate */
	 idf= -n;
	 dtb=(lgbcsg.pagmar[1]-lgbcsg.pagmar[0])/idf;
	 for(i=0;i<idf;i++) lgbcsa.tbor[i]=lgbcsg.pagmar[0]+i*dtb;
      }
      else
      {  
/* n>0 : Posizioni definite dall'utente */
	 dtb=lgbcsg.pagmar[0];
	 for(i=0;i<n;i++)
	 {  idf=i;
	    if(tab[i]==0.) break;
	    if(tab[i]<dtb || tab[i]>lgbcsg.pagmar[1]) goto errore;
	    dtb=tab[i];
	 }
	 for(i=0;i<idf;i++) lgbcsa.tbor[i]=tab[i];
      }
/* Completamento delle posizioni restanti */
      if(idf>0 && idf<MIZ_TABOR-1)
      {  for(i=idf;i<MIZ_TABOR;i++)
	 	lgbcsa.tbor[i]=lgbcsa.tbor[i-1];
      }
   }
#if MIZ_FILE
   lgwtbo();
#endif
   return;
errore:
   lgerr(LGE_TABULAZ);
   return;
}
/* Sottoprogramma deftbv */
VOID deftbv(DOUBLE tab[],int n)
{  if((ABS(n))>MIZ_TABVER) goto errore;
   if(n==0)
   {  int i;
/* Azzeramento delle tabelle di tabulazione */
      for(i=0;i<MIZ_TABVER;lgbcsa.tbver[i++]=0.);
   }
   else
   {  int idf,i;
      double dtb;
      if(n<0)
      {  
/* n<0 : Riempimento delle prime -N posizioni in modo che 
   siano equidistanziate */
	 idf= -n;
	 dtb=(lgbcsg.pagmar[3]-lgbcsg.pagmar[2])/idf;
	 for(i=0;i<idf;i++) lgbcsa.tbver[i]=lgbcsg.pagmar[3]-i*dtb;
      }
      else
      {  
/* n>0 : Posizioni definite dall'utente */
	 dtb=lgbcsg.pagmar[3];
	 for(i=0;i<n;i++)
	 {  idf=i;
	    if(tab[i]==0.) break;
	    if(tab[i]>dtb || tab[i]<lgbcsg.pagmar[2]) goto errore;
	    dtb=tab[i];
	 }
	 for(i=0;i<idf;i++) lgbcsa.tbver[i]=tab[i];
      }
/* Completamento delle posizioni restanti */
      if(idf>0 && idf<MIZ_TABVER-1)
      {  for(i=idf;i<MIZ_TABVER;i++)
	 	lgbcsa.tbver[i]=lgbcsa.tbver[i-1];
      }
   }
#if MIZ_FILE
   lgwtbv();
#endif
   return;
errore:
   lgerr(LGE_TABULAZ);
   return;
}
/* Sottoprogramma defpen */
VOID defpen(kl,kf)
int kl,kf;
{  
#if MIZ_INTERATTIVA
   if(kl<=0 || kl>MIZ_NUMPEN)
   {  lgerr(LGE_PENNA);
      return;
   }
   if(kf<0)
   {  lgbcpn.kpenna[kl-1]=lggraf.knpen+1;
   }
   if(kf==0)
   {  lgbcpn.kpenna[kl-1]=lggraf.knpen+1;
   }
   if(kf>0)
   {  lgbcpn.kpenna[kl-1]=1+(kf-1)%lggraf.knpen;
   }
#endif
   return;
}
/* Sottoprogramma defpns */
VOID defpns(int kl)
{  
#if MIZ_INTERATTIVA
   if(kl>0 && kl<=MIZ_NUMPEN)
   {  lgbcsg.kpnsim=kl;
   }
   else
   {  lgerr(LGE_PENNA);
   }
#endif
   return;
}
/* Sottoprogramma defpun */
VOID defpun(int is,double dim,double rot)
{  
#if MIZ_INTERATTIVA
   if(is<=0)
   {  lggraf.punsim=FALSE;
   }
   else
   {  lggraf.punsim=TRUE;
      if(is<=MIZ_CARMX)
      {  lgbccf.kspun=is;
	 if(dim>0.)
	 {  lgbccf.dimpun=MIN(dim,MIZ_DSIMX);
	    if(lgbccf.dimpun!=dim) lgerr(LGE_DIMCARATT);
	 }
	 if(rot>=0.)
	 {  lgbccf.rotpun=fmod(lgbccf.rotpun,lgbccs.angiro);
	    if(!lgbcge.kanrad) lgbccf.rotpun*=lgbccs.grarad;
	    lgbccf.copun=cos(lgbccf.rotpun);
	    lgbccf.sepun=sin(lgbccf.rotpun);
	 }
      }
      else
      {  lgerr(LGE_SIMBOLOCENT);
      }
   }
#endif
   return;
}
/* Sottoprogramma defris */
VOID defris(int k)
{  int j;
#if MIZ_INTERATTIVA
   j=ABS(k);
   if(j>2)
   {  lgerr(LGE_RISCALAM);
      return;
   }
   lgbccf.kmprz=2*j;
   if(k<0) lgbccf.kmprz=lgbccf.kmprz+1;
   lgbccf.kcgfdf=FALSE;
#endif
   return;
}
/* Sottoprogramma defrot */
VOID defrot(double ang)
{  
#if MIZ_INTERATTIVA
   lgbccf.rotgf=fmod(ang,lgbccs.angiro);
   if(!lgbcge.kanrad) lgbccf.rotgf*=lgbccs.grarad;
   lgbccf.crotgf=cos(lgbccf.rotgf);
   if((ABS(lgbccf.crotgf))<1.e-5) lgbccf.crotgf=0.;
   lgbccf.srotgf=sin(lgbccf.rotgf);
   if((ABS(lgbccf.srotgf))<1.e-5) lgbccf.srotgf=0.;
   lgbccf.kcgfdf=FALSE;
#endif
   return;
}
/* Sottoprogramma defsch */
VOID defsch(double fxmin,double fxmax,double fymin,double fymax)
{  
#if MIZ_INTERATTIVA
   if(fxmin>=0. && fxmin<fxmax && fxmax<=lggraf.dlim[0]
   	&& fymin>=0. && fymin<fymax && fymax<=lggraf.dlim[1])
   {  lgbccf.flim[0]=fxmin;
      lgbccf.flim[1]=fxmax;
      lgbccf.flim[2]=fymin;
      lgbccf.flim[3]=fymax;
      lgbccf.kmprz=ABS(lgbccf.kmprz);
      lgbccf.kcgfdf=FALSE;
   }
   else
   {  lgerr(LGE_AREAFISICA);
   }
#endif
   return;
}
/* Sottoprogramma deferr */
VOID deferr(int k,int liv,int mess,int conta)
{ 
  int coderr,livpre;
  if(k<0 || k>MIZ_NUMMSGERR) {
    lgerr(LGE_NONCODERR);
    return;
  }
  coderr=lgbcer.errve[k-1].stato&EST_NUMMSGERR;
  livpre=lgbcer.errve[k-1].stato|EST_CODERR;
  if(livpre==MSG_C_FATALE && liv!=MSG_FATALE) {
    lgerr(LGE_DEFERR);
    return;
  }
  switch(liv) {
  case MSG_WARNING:
    coderr|=MSG_C_WARNING;
    break;
  case MSG_MAXCONT:
    coderr|=MSG_C_MAXCONT;
    coderr|=EST_CONTAERR;
    lgbcer.errve[k-1].cont=0;
    break;
  case MSG_ERRORE:
    coderr|=MSG_C_ERRORE;
    break;
  case MSG_FATALE:
    coderr|=MSG_C_FATALE;
    if(mess==0) lgerr(LGE_STFATALDIS);
    break;
  }
  if(mess>0) coderr|=EST_STAMPAMSG;
  else if(mess<0)  coderr|=EST_STPRIMOER;
  if(conta!=0)  coderr|=EST_CONTAERR;
  lgbcer.errve[k-1].stato=coderr;
  return;
}
/* Sottoprogramma defdim */
VOID defdim(int jd)
{
#if MIZ_INTERATTIVA
  if(jd<=0 || jd>10) lgerr(LGE_INVDIM);
  else {
    jd--;
    lggraf.klimpg=jd;
    lggraf.dlim[0]=lggraf.dlimps[jd][0];
    lggraf.dlim[1]=lggraf.dlimps[jd][1];
  }
#endif
  return;
}
/* Sottoprogramma defqlt */
VOID defqlt(int jq)
{
#if MIZ_INTERATTIVA
  if(jq < lggraf.qltmin || jq > lggraf.qltmax) 
    lgerr(LGE_INVQLT);
  else
    lggraf.quality = jq;
#endif
  return;
}
