/* Interprete per emulatore Tektronix 4010 */
#include <stdio.h>
#include <termios.h>
#include <unistd.h>
#include "mizlocdf.h"
#include "mizmsg.h"
#include "mizcmn.h"
#include "icpcmn.h"
/*  */
#define GXMAX 1023
#define GYMAX 779
#define TEK_LUNBUF 504
#define HY '\040'
#define HX '\040'
#define LY '\140'
#define LX '\100'
#define GS '\035'
#define US '\037'
#define ESC '\033'
#define SB  '\032'
/*  */
#define ABS(z) ((z)>=0)? (z):-(z)
#define MAX(z,w) ((z)>=(w))? (z):(w)
#define MIN(z,w) ((z)<=(w))? (z):(w)
/*  */
union mizhdr mizhdr;
extern struct lgbcio lgbcio;
extern struct lggraf lggraf;
/*  */
struct {
  int kxult;		          /* Ultime coordinate fisiche. */
  int kyult;
  int koutb;
  int kcb;
  char iniseq[2];
  char lpun[4];
  char lbuf[10];
  char loutbf[TEK_LUNBUF+8];        /* Buffer di output. */
} lgctek={0,0,0,0,
          ESC,FF,HY,LY,HX,LX};
/*  */
/* Sottoprogramma lgsvuo -- scarica il buffer corrente */
VOID lgsvuo()
{
  lgctek.loutbf[lgctek.koutb++]=US;
  lgctek.loutbf[lgctek.koutb++]=NUL;
  fputs(lgctek.loutbf,lgbcio.kunoug);
  fflush(lgbcio.kunoug);
  lgctek.loutbf[0]=GS;
  lgctek.loutbf[1]=lgctek.lpun[3];
  lgctek.koutb=2;
  return;
}/*  */
/* Sottoprogramma lgbuf - Bufferizzazione delle sequenze grafiche */
VOID lgbuf(l,n,modo)
char l[1];
int n;
int modo;
{
  int i;
  if(n>0) {
    if(n>TEK_LUNBUF-lgctek.koutb) lgsvuo();
    for(i=0;i<n;lgctek.loutbf[lgctek.koutb++]=l[i++]);
  }
  if(modo<0) lgsvuo();
  return;
}
/*  */
/* Sottoprogramma LGCUR */
/* Questo sottoprogramma e' previsto solo per le versioni interattive.
   Qui serve solo per evitare errori di "undefined external" in fase di 
   link. 
   Il sottoprogramma per l'interprete Tektronix e' ancora da scrivere.
*/
CHAR lgcur(fx,fy)
DOUBLE *fx,*fy;
{  
#if MIZAR_LinuxVERSION
  struct termios termios_p,termios_new_p;
  int j,kx,ky;
  char gin[8];
  lgsvuo();
				/* get current terminal attributes */
  tcgetattr(STDIN_FILENO,&termios_new_p);
  termios_p = termios_new_p;    /* and make a copy */
  termios_new_p.c_lflag &= !(ICANON);
				/* set terminal unbuffered, noecho */
  tcsetattr ( STDIN_FILENO, TCSAFLUSH, &termios_new_p);
  putchar(ESC);  putchar(SB); 	/* Enable GIN mode */
  for(j=0; j<5; j++) gin[j] = getchar();  
				/* reset terminal attributes */
  tcsetattr ( STDIN_FILENO, TCSAFLUSH, &termios_p);
  kx = gin[1]&037; kx <<= 5; kx |= (gin[2]&037);
  ky = gin[3]&037; ky <<= 5; ky |= (gin[4]&037);
  *fx = kx;
  *fy = ky;
  return(gin[0]);
#else
  *fx=lgctek.kxult;
  *fy=lgctek.kyult;
  return(CR);
#endif 
}
/*  */
/* Sottoprogramma LGZOOM */
/* Questo sottoprogramma e' previsto solo per le versioni interattive.
   Qui serve solo per evitare errori di "undefined external" in fase di 
   link. 
   Il sottoprogramma per l'interprete Tektronix e' ancora da scrivere.
*/
CHAR lgzoom(fx0,fy0,fx1,fy1)
DOUBLE *fx0,*fy0,*fx1,*fy1;
{  *fx0=0.;
   *fy0=lggraf.dflim[0];
   *fx1=0.;
   *fy1=lggraf.dflim[1];
   return(CR);
}
/*  */
/* Sottoprogramma lgmov -- codifica delle coordinate secondo il 
   protocollo Tektronix 4010 */
VOID lgmov(fx,fy)
double fx,fy;
{
  char l[4];
  lgctek.kxult=fx;
  lgctek.kyult=fy;
  l[0]=lgctek.kyult>>5;
  l[0]|=HY;
  l[1]=lgctek.kyult&0x1F;
  l[1]|=LY;
  l[2]=lgctek.kxult>>5;
  l[2]|=HX;
  l[3]=lgctek.kxult&0x1F;
  l[3]|=LX;
  if(l[0]!=lgctek.lpun[0]) 
    lgctek.lbuf[lgctek.kcb++]=lgctek.lpun[0]=l[0];
  if(l[2]!=lgctek.lpun[2]) {
    lgctek.lbuf[lgctek.kcb++]=lgctek.lpun[1]=l[1];
    lgctek.lbuf[lgctek.kcb++]=lgctek.lpun[2]=l[2];
  }
  else if(l[1]!=lgctek.lpun[1]) {
    lgctek.lbuf[lgctek.kcb++]=lgctek.lpun[1]=l[1];
  }
  lgctek.lbuf[lgctek.kcb++]=l[3];
  lgbuf(lgctek.lbuf,lgctek.kcb,0);
  lgctek.lpun[3]=l[3];
  lgctek.kcb=0;
  lggraf.kstpn=ABS(lggraf.kstpn);    /* Lo spostamento provoca
					l'abbassamento della penna */
  return;
}
/*  */
/* Sottoprogramma lgpag */
VOID lgpag(kpg)
int kpg;
{
  int kpg1;
  int i;
  kpg1=kpg; if(kpg1>0) kpg1=ICP_PAGINA;
  switch (kpg1) {
  case ICP_INIT:
    /*    lgbcio.kunoug=fopen(lggraf.nomoug,"w"); */
    lgbcio.kunoug=stdout;
    lgctek.loutbf[0]=GS;
    for(i=0;i<TEK_LUNBUF;lgctek.loutbf[++i]=US);
    lgctek.loutbf[TEK_LUNBUF+1]=NUL;
    break;
  case ICP_PAGINA:
    lgbuf(lgctek.iniseq,2,-1);
    break;
  case ICP_FINE:
    lgsvuo();
    lgbuf(lgctek.iniseq,2,-1);
    lgsvuo();
    break;
  case ICP_SVUOTA:
    lgsvuo();
    break;
  case ICP_INIDATA:
    fseek(lgbcio.kunmiz,mizhdr.hdr.bldtek,SEEK_SET);
    fread(&lggraf,sizeof(struct lggraf),1,lgbcio.kunmiz);
    break;
  }
  return;
}
/* Sottoprogramma LGPEN */
VOID lgpen(k)
int k;
{  
  if(lggraf.kstpn!=k) {
    lggraf.kstpn=k;
    if(k>=0) {
      lgctek.lbuf[0]=lgctek.lpun[3];
      lgbuf(lgctek.lbuf,1,0);
    }
    else {
      lgctek.lbuf[lgctek.kcb++]=GS;
    }
  }
  return;
}
